% ECMPR DEMO call ECMPR to register point set X and Y  visualize the result

clc
clear
close all

% Point sets are stored as N x 3 "matrices" in .txt files "pointCloudX.txt"
% and "pointCloudY.txt" respectively

% FORMAT, each file is simply a whitespace separated file with arbitary rows
% and 3 columns where each row contains the x y z coordinates of a 3D point.
X = dlmread('pointCloudX.txt',' ')';
Y = dlmread('pointCloudY.txt',' ')';

% due to the high percentage of outliers, some extra iterations are needed.
maxNumIter = 70;

% call ecmpr() to do the actual compuation
[R,t,v,a,Thistory] = ecmpr(X,Y,maxNumIter);

% define colors and markers
clrx = [0 124 175]/255; % color for the model points, X (GMM).
clry = [1 0 0]; % color for the data points, Y.
markerSize = .7; % "dot" size for the points
marker = '.';
bgcolor = +.1+[.52 .57 .57]; % background color

% set some axis limits based on the final alginment plus a margin
TX = bsxfun(@plus,R*X,t);
lockAxes = [min(TX(1,:)) max(TX(1,:)) min(TX(2,:)) max(TX(2,:)) min(TX(3,:)) max(TX(3,:))] + [-.04 .04 -.04 .04 -.04 .04];




% show the initial position of X and Y
figure(1);
axes1 = gca;
hold on
title('Initial poisition of X and Y.','fontweight','bold','fontsize',12);
axis(lockAxes)

% set the legend
plot(NaN,'marker','o','markeredgecolor',clrx,'markersize',11,'linestyle','none','markerfacecolor',clrx);
plot(NaN,'marker','o','markeredgecolor',clry,'markersize',11,'linestyle','none','markerfacecolor',clry);
h1 = legend('X [GMM]','Y (data)');
set(h1,'fontweight','bold');

% show the points
plot3(X(1,:),X(2,:),X(3,:),'markersize',markerSize,'marker','.','linestyle','none','color',clrx)
plot3(Y(1,:),Y(2,:),Y(3,:),'markersize',markerSize,'marker','.','linestyle','none','color',clry)

% "lock" axes and viewing angle to be the same on all figures
set(axes1,'Xticklabel',[],'Yticklabel',[],'Zticklabel',[],'color',bgcolor);
grid on
set(1,'color',[1 1 1],'position',get(1,'position')+[-580 0 0 0]);
view([48 14])
hold off




% Show the registration after every iteration
figure(2);
for iter = 1:maxNumIter
    
    % apply current transformation
    TX = bsxfun(@plus,Thistory{iter,1}*X,Thistory{iter,2});
    
    % clear old data
    clf(2);
    
    hold on
    title(sprintf('Registration of X and Y after %d iterations.\n',iter),'fontweight','bold','fontsize',12);
    axis(lockAxes)
    
    % set the legend
    plot(NaN,'marker','o','markeredgecolor',clrx,'markersize',11,'linestyle','none','markerfacecolor',clrx);
    plot(NaN,'marker','o','markeredgecolor',clry,'markersize',11,'linestyle','none','markerfacecolor',clry);
    h1 = legend('T(X) [GMM]','Y (data)');
    set(h1,'fontweight','bold');
    
    % show the points
    plot3(TX(1,:),TX(2,:),TX(3,:),'markersize',markerSize,'marker','.','linestyle','none','color',clrx)
    plot3(Y(1,:),Y(2,:),Y(3,:),'markersize',markerSize,'marker','.','linestyle','none','color',clry)
    
    % modify viewing angle and axis
    set(gca,'Xticklabel',[],'Yticklabel',[],'Zticklabel',[],'color',bgcolor);
    set(2,'color',[1 1 1]);
    grid on
    view([48 14])
    hold off
    pause(.1);
end



